package com.copperarrow.example.wickettesting;

import com.copperarrow.example.wickettesting.view.CustomerView;
import org.apache.wicket.cdi.CdiConfiguration;
import org.apache.wicket.markup.html.WebPage;
import org.apache.wicket.protocol.http.WebApplication;
import org.jboss.vfs.VirtualFile;
import org.reflections.ReflectionsException;
import org.reflections.vfs.SystemDir;
import org.reflections.vfs.Vfs;
import org.reflections.vfs.ZipDir;
import org.wicketstuff.javaee.injection.JavaEEComponentInjector;
import org.wicketstuff.javaee.naming.global.ModuleJndiNamingStrategy;

import javax.enterprise.inject.spi.BeanManager;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import java.io.IOException;
import java.net.URL;
import java.util.jar.JarFile;

import static org.apache.wicket.cdi.ConversationPropagation.NONE;

/**
 * Application object for your web application. If you want to run this
 * application without deploying, run the Start class.
 *
 * @see com.copperarrow.example.wickettesting.Start#main(String[])
 */
public class WicketApplication extends WebApplication {

    /**
     * @see org.apache.wicket.Application#getHomePage()
     */
    @Override
    public Class<? extends WebPage> getHomePage() {
        return HomePage.class;
    }

    /**
     * @see org.apache.wicket.Application#init()
     */
    @Override
    public void init() {
        super.init();

        configureVFS();
        configureCDIEJB();

        mountPage("/customerview", CustomerView.class);
    }

    private void configureVFS() {
        Vfs.addDefaultURLTypes(
                new Vfs.UrlType() {
                    public boolean matches(URL url) {
                        return url.getProtocol().equals("vfs");
                    }

                    public Vfs.Dir createDir(URL url) {
                        VirtualFile content;
                        try {
                            content = (VirtualFile) url.openConnection().getContent();
                        } catch (Throwable e) {
                            throw new ReflectionsException("could not open url connection as VirtualFile [" + url + "]", e);
                        }

                        Vfs.Dir dir = null;
                        try {
                            dir = createDir(new java.io.File(content.getPhysicalFile().getParentFile(), content.getName()));
                        } catch (IOException e) { /*continue*/ }
                        if (dir == null) {
                            try {
                                dir = createDir(content.getPhysicalFile());
                            } catch (IOException e) { /*continue*/ }
                        }
                        return dir;
                    }

                    Vfs.Dir createDir(java.io.File file) {
                        try {
                            return file.exists() && file.canRead() ? file.isDirectory() ? new SystemDir(file) : new ZipDir(new JarFile(file)) : null;
                        } catch (IOException e) {
                            e.printStackTrace();
                        }
                        return null;
                    }
                });
    }

    private void configureCDIEJB() {
        getComponentInstantiationListeners().add(new JavaEEComponentInjector(this, new ModuleJndiNamingStrategy()));
        // Enable CDI
        BeanManager bm;
        try {
            bm = (BeanManager) new InitialContext().lookup("java:comp/BeanManager");
        } catch (NamingException e) {
            throw new IllegalStateException("Unable to obtain CDI BeanManager", e);
        }

        // Configure CDI, disabling Conversations as we aren't using them
        new CdiConfiguration(bm).setPropagation(NONE).configure(this);
    }
}
