/*
 * Copyright 2013 dbeer.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.copperarrow.example.wickettesting.view;

import com.copperarrow.example.wickettesting.dao.CustomerFacade;
import com.copperarrow.example.wickettesting.entities.Customer;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.Iterator;
import java.util.List;
import javax.inject.Inject;
import org.apache.wicket.cdi.CdiContainer;
import org.apache.wicket.extensions.markup.html.repeater.data.sort.SortOrder;
import org.apache.wicket.extensions.markup.html.repeater.util.SortableDataProvider;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.Model;

/**
 *
 * @author dbeer
 */
class CustomerSortableDataProvider extends SortableDataProvider<Customer, String> {

    @Inject
    private CustomerFacade dvcsDAO;

    public CustomerSortableDataProvider() {
        CdiContainer.get().getNonContextualManager().inject(this);
        setSort("repositoryName", SortOrder.NONE);
    }

    @Override
    public Iterator<? extends Customer> iterator(long first, long count) {
        List<Customer> data = new ArrayList<Customer>(dvcsDAO.findRange((int) first, (int) count));
        Collections.sort(data, new Comparator<Customer>() {
            @Override
            public int compare(Customer o1, Customer o2) {
                if (getSort() != null) {
                    int dir = getSort().isAscending() ? 1 : -1;
                    if ("repositoryName".equals(getSort().getProperty())) {
                        return dir * (o1.getName().compareTo(o2.getName()));
                    }
                }
                return 0;
            }
        });
        return data.subList((int) first,
                (int) Math.min(first + count, dvcsDAO.count())).iterator();
    }

    @Override
    public long size() {
        return dvcsDAO.count();
    }

    @Override
    public IModel<Customer> model(Customer object) {
        return Model.of(object);
    }

}
