/*
 * Copyright 2013 dbeer.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.copperarrow.example.wickettesting.view;

import com.copperarrow.example.wickettesting.dao.CustomerFacade;
import com.copperarrow.example.wickettesting.entities.Customer;
import java.util.ArrayList;
import java.util.List;
import javax.inject.Inject;
import org.apache.wicket.extensions.markup.html.repeater.data.table.DefaultDataTable;
import org.apache.wicket.extensions.markup.html.repeater.data.table.IColumn;
import org.apache.wicket.extensions.markup.html.repeater.data.table.PropertyColumn;
import org.apache.wicket.markup.html.WebPage;
import org.apache.wicket.markup.html.form.Button;
import org.apache.wicket.markup.html.form.Form;
import org.apache.wicket.markup.html.form.TextField;
import org.apache.wicket.model.Model;
import org.apache.wicket.model.PropertyModel;
import org.apache.wicket.request.mapper.parameter.PageParameters;

/**
 *
 * @author dbeer
 */
public final class CustomerView extends WebPage {

    @Inject
    Customer customer;

    @Inject
    CustomerFacade dao;

    public CustomerView(PageParameters params) {

        List<IColumn<Customer, String>> columns = new ArrayList<IColumn<Customer, String>>();
        columns.add(new PropertyColumn<Customer, String>(Model.of("Name"), "name", "name"));
        columns.add(new PropertyColumn<Customer, String>(Model.of("Surname"), "surname"));

        DefaultDataTable<Customer, String> table = new DefaultDataTable<Customer, String>("table", columns, new CustomerSortableDataProvider(), 10);
        add(table);

        addform();
    }

    private void addform() {
        Form form = new Form("form") {

            @Override
            protected void onSubmit() {
                System.out.println("form submit");
                dao.create(customer);
                setResponsePage(CustomerView.class);
            }

        };
        add(form);
        form.add(new TextField("name", new PropertyModel(customer, "name")));
        form.add(new TextField("surname", new PropertyModel(customer, "surname")));
        form.add(new Button("submit", Model.of("Submit")));
    }
}
