/*
 * Copyright 2013 dbeer.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.copperarrow.example.wickettesting;

import java.io.File;
import org.apache.wicket.Application;
import org.apache.wicket.mock.MockApplication;
import org.apache.wicket.protocol.http.WebApplication;
import org.apache.wicket.resource.loader.ClassStringResourceLoader;
import org.apache.wicket.util.tester.WicketTester;
import org.jboss.arquillian.container.test.api.Deployment;
import org.jboss.arquillian.testng.Arquillian;
import org.jboss.shrinkwrap.api.Archive;
import org.jboss.shrinkwrap.api.ShrinkWrap;
import org.jboss.shrinkwrap.api.spec.WebArchive;
import org.jboss.shrinkwrap.resolver.api.DependencyResolvers;
import org.jboss.shrinkwrap.resolver.api.maven.MavenDependencyResolver;
import org.testng.annotations.AfterMethod;
import org.testng.annotations.BeforeMethod;
import org.testng.annotations.Test;

/**
 *
 * @author dbeer
 */
public class HomePageNGTest extends Arquillian {

    static MavenDependencyResolver resolver = DependencyResolvers.use(
            MavenDependencyResolver.class).loadMetadataFromPom("pom.xml");

    private WicketTester wicketTester;

    public HomePageNGTest() {
    }

    @Deployment
    public static Archive<?> createDeployment() {
        WebArchive war = ShrinkWrap.create(WebArchive.class, "test.war");
        File[] libs = resolver.includeDependenciesFromPom("pom.xml").scope("test").resolveAsFiles();
        for (File file : libs) {
            war.addAsLibrary(file);
        }

        war.addClasses(HomePage.class, HomePageNGTest.class); 
        System.out.println(war.toString(true));
        return war;
    }

    @BeforeMethod
    public void setUp() throws Exception {
        wicketTester = new WicketTester(getWebApplication());
        // trick to make Wicket tests use properties files named like the test as first resource for translation
        wicketTester.getApplication().getResourceSettings().getStringResourceLoaders().add(0, new ClassStringResourceLoader(getClass()));
        wicketTester.getApplication().getResourceSettings().getStringResourceLoaders().add(1, new ClassStringResourceLoader(getApplicationClass()));
    }

    protected Class<? extends Application> getApplicationClass() {
        return MockApplication.class;
    }

    @AfterMethod
    public void tearDown() {
        wicketTester.getApplication().internalDestroy();
    }

    protected WebApplication getWebApplication() {
        return new MockApplication();
    }

    @Test
    public void testSomeMethod() {
        //start and render the test page
        wicketTester.startPage(HomePage.class);

        //assert rendered page class
        wicketTester.assertRenderedPage(HomePage.class);
    }

}
